use std::{fs, io, path::Path};

use serde::{Deserialize, Serialize};
use sqlx::SqlitePool;
use thiserror::Error;

#[derive(Debug, Clone, Serialize, Deserialize)]
pub struct Config {
    pub host: String,
    pub port: u16,
    pub password: String,
    pub videos_per_page: usize,
}

impl Default for Config {
    fn default() -> Self {
        Self {
            host: "0.0.0.0".into(),
            port: 3000,
            password: "123456".into(),
            videos_per_page: 10,
        }
    }
}

/// This controls an instance and its state.
#[derive(Debug, Clone)]
pub struct Instance {
    pub config: Config,
    pub pool: SqlitePool,
}

#[derive(Debug, Error)]
pub enum NewInstanceError {
    #[error("Failed to open TOML configuration file")]
    ConfigLoad(#[from] io::Error),
    #[error("Could not parse TOML configuration")]
    ConfigParse(#[from] toml::de::Error),
    #[error("Failed to create connection pool")]
    PoolConnect(#[from] sqlx::Error),
}

impl Instance {
    pub async fn new<P>(config_file: P, database_url: &str) -> Result<Self, NewInstanceError>
    where
        P: AsRef<Path>,
    {
        let config = toml::from_str(&fs::read_to_string(config_file)?)?;
        let pool = SqlitePool::connect(database_url).await?;

        Ok(Self { config, pool })
    }
}
